package com.sillycycle.bagleyd.panex;

/*
 * @(#)PanexFrame.java
 *
 * Copyright 1994 - 2023  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sillycycle.bagleyd.panex.model.PanexFormat;
import com.sillycycle.bagleyd.util.ArgumentParser;
import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.ComponentUtil;
import com.sillycycle.bagleyd.util.Icon;
import com.sillycycle.bagleyd.util.IODialog;
import com.sillycycle.bagleyd.util.OKDialog;
import com.sillycycle.bagleyd.util.Spinner;

/**
 * The <code>PanexFrame</code> class is to run as an application.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

public class PanexFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = 42L;
	public static final String NL = System.getProperty("line.separator");
	static final int MAX_TILES = 10;
	static final int MIN_SPEED = 1;
	static final int MAX_SPEED = 50;
	static final int NEVER = -1;
	static final String SCOREFILE = "panex.scores";
	static final String[] COMMAND_STRINGS = new String[] {
		"Get Buffer", "Write Buffer", "Show Buffer",
		"Undo", "Redo", "Clear", "Auto-solve",
		"Mode", "Increment", "Decrement",
		"Speed >", "Slow <", "Sound @",
		"Description ?", "Features !", "References ^", "About",
		"Get File", "Write File", "Exit"};
	static final char[] COMMAND_CHARS = new char[] {
		'g', 'w', '*',
		'u', 'r', 'c', 's',
		'm', 'i', 'd', '>', '<', '@',
		'?', '!', '^', 'a', 'G', 'W', 'x'};

	static ArgumentParser argumentParser = null;
	Icon mediumIcon = null;
	Icon largeIcon = null;
	int widthValue = 340;
	int heightValue = 255;
	static int xValue = 0;
	static int yValue = 0;
	String monoString = "MO";
	Color[] pyramidColor = new Color[] {
		Color.magenta, Color.orange
	};
	Color tileColor = Color.lightGray;
	Color bgColor = Color.darkGray;
	boolean sound = false;
	boolean controlValue = true;
	int tileValue = PanexFormat.DEFAULT_TILES;
	int modeValue = PanexFormat.defaultMode.ordinal();
	int delayValue = 10;
	int speedValue = MAX_SPEED + MIN_SPEED - 1 - delayValue;
	boolean debug = false;
	JPanel topPanel = new JPanel();
	JComboBox<Object> modeComboBox = new JComboBox<>();
	Panex puzzle;
	int movesDisplay = 0;
	String recordDisplay = "NEVER";
	String messageDisplay = "     Play Panex!                           ";
	JTextField movesValueLabel = new JTextField(Integer.toString(
		movesDisplay));
	JTextField recordValueLabel = new JTextField(recordDisplay);
	Label messageLabel = new Label(messageDisplay, Label.CENTER);
	// JLabel constantly resizes
	OKDialog aDialog = null;
	StringBuilder stringBuilderSave;
	IODialog ioDialog = null;
	int[][] puzzleRecord = null;
	JMenuBar menuBar;
	JPanel mainPanel = new JPanel();
	JPanel secondaryPanel = new JPanel();
	JFrame frame;
	JCheckBoxMenuItem soundMenuItem = null;
	Spinner tileChanger = null, speedChanger = null;
	//Slider tileChanger = null, speedChanger = null;

	public String writePuzzleString() {
		return this.stringBuilderSave.toString();
	}

	public void getPuzzleString(String s) {
		puzzle.stringSave = s;
	}

	public void processKey(char key) {
		puzzle.processKey(key);
	}

	public boolean getToggleSound() {
		return sound;
	}

	public void toggleSound() {
		sound = !sound;
		if (controlValue)
			soundMenuItem.setSelected(sound);
	}

	public String getOpt(String string) {
		return argumentParser.getOption(string);
	}

	public int getOpt(String param, int value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public boolean getOpt(String param, boolean value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			if (string.equalsIgnoreCase("true") ||
					string.equalsIgnoreCase("yes") ||
					string.equalsIgnoreCase("on"))
				return true;
			else if (string.equalsIgnoreCase("false") ||
					string.equalsIgnoreCase("no") ||
					string.equalsIgnoreCase("off"))
				return false;
			return (Integer.parseInt(string) != 0);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public Color getOpt(String param, Color value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Color.decode(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public void getOpts() {
		widthValue = getOpt("windowWidth", widthValue);
		heightValue = getOpt("windowHeight", heightValue);
		xValue = getOpt("windowX", xValue);
		yValue = getOpt("windowY", yValue);
		for (int i = 0; i < pyramidColor.length; i++)
			pyramidColor[i] = getOpt("pyramidColor" + i,
				pyramidColor[i]);
		tileColor = getOpt("tileColor", tileColor);
		bgColor = getOpt("bgColor", bgColor);
		sound = getOpt("sound", sound);
		controlValue = getOpt("control", controlValue);
		tileValue = getOpt("tiles", tileValue);
		if (tileValue < PanexFormat.MIN_TILES)
			tileValue = PanexFormat.DEFAULT_TILES;
		modeValue = getOpt("mode", modeValue);
		if (modeValue < PanexFormat.Modes.HANOI.ordinal() ||
				modeValue >= PanexFormat.Modes.values().length)
			modeValue = PanexFormat.defaultMode.ordinal();
		delayValue = getOpt("delay", delayValue);
	}

	public void showBuffer() {
		if (ioDialog != null) {
			ioDialog.setVisible(true);
		}
	}

	public void showMessage(String msg) {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Panex", msg, "OK",
				mediumIcon, false);
			aDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2, 350, 120);
			aDialog.setVisible(true);
		}
	}

	public void descriptionHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Description",
"Panex - A grooved sliding tile puzzle created by Toshio" + NL +
"Akanuma and manufactured by the Tricks Co., Ltd of Tokyo," + NL +
"Japan (a Magic Company) in the 1980's.  Mathematicians at" + NL +
"Bell Laboratories calculated the number of moves to be 27,564" + NL +
"to 31,537.  It came in two varieties: one with a magenta and" + NL +
"a orange pyramid of order 10 on silver tiles; in the gold" + NL +
"version pieces of each color look alike (i.e. no pyramid is" + NL +
"drawn on them), this is a little harder.  The goal in this" + NL +
"puzzle is to simply exchange the 2 piles.  Pieces with" + NL +
"smaller trapazoids cannot go down as far as pieces with" + NL +
"bigger trapazoids." + NL +
" " + NL +
"The original Tower of Hanoi puzzle is the invention of" + NL +
"Edouard Lucas and was sold as a toy in France in 1883.  The" + NL +
"legend of 64 disks in the great temple of Benares of the god" + NL +
"Brahma is also his invention.  The goal in this puzzle is to" + NL +
"move the pile from the left side to the right most column." + NL +
"Unlike panex, a large trapazoid cannot go on top of a smaller" + NL +
"one, but pieces always fall to the bottom." + NL +
" " + NL +
"The original Algorithme 6 is 2 stacks of 3 wooden spheres on" + NL +
"2 of 3 posts.  The spheres come in 3 different sizes.  The goal" + NL +
"goal is to swap the spheres using the posts without putting a" + NL +
"bigger sphere on a smaller one and without exceeding the" + NL +
"size of the post.  It was created and produced by Patrick" + NL +
"Farvacque around 1997.  The puzzle presented here has a" + NL +
"simpler solution because the tiles are all the same height" + NL +
"(i.e. a 39 move solution as opposed to 66).",
					"OK", largeIcon, false);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void featuresHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Features",
"Press \"mouse-left\" button to move a tile the top tile in a" + NL +
"column.  Release \"mouse-left\" button on another column. It" + NL +
"will not move if blocked." + NL +
" " + NL +
"Double click on \"mouse-right\" button or press \"C\" or \"c\" keys," + NL +
"clear the puzzle." + NL +
" " + NL +
"Press \"g\" key to get (read) a saved puzzle to screen." + NL +
" " + NL +
"Press \"w\" key to save (write) a puzzle to screen." + NL +
" " + NL +
"Press \"G\" key to get (read) a saved puzzle to file." + NL +
" " + NL +
"Press \"W\" key to save (write) a puzzle to file." + NL +
" " + NL +
"Press \"U\" or \"u\" keys to undo a move." + NL +
" " + NL +
"Press \"R\" or \"r\" keys to redo a move." + NL +
" " + NL +
"Press \"S\" or \"s\" keys to auto-solve.  Unfortunately, its" + NL +
"only implemented from the starting position." + NL +
" " + NL +
"Press \"M\" or \"m\" keys to switch between Hanoi (one pyramid" + NL +
"column), Algorithme, and Panex, (each has two pyramid" + NL +
"columns) modes (they each have different rules).  In Hanoi," + NL +
"one cannot place larger trapezoid on a smaller trapezoid." + NL +
"Here the goal is to move the pile from the left peg to the" + NL +
"rightmost peg.  Algorithme is similar, here we must exchange" + NL +
"tiles and we are limited by the size of the stack.  A move" + NL +
"from stack 1 to stack 3 and vice-versa when stack 2 is full." + NL +
"In Panex, a tile cannot go lower that its initial starting" + NL +
"point.  Here again, the goal is to exchange the 2 piles." + NL +
" " + NL +
"Press \"I\" or \"i\" keys to increase the number of tiles." + NL +
" " + NL +
"Press \"D\" or \"d\" keys to decrease the number of tiles." + NL +
" " + NL +
"Press \">\" or \".\" keys to speed up the movement of the tiles." + NL +
" " + NL +
"Press \"<\" or \",\" keys to slow up the movement of the tiles." + NL +
" " + NL +
"Press \"@\" key to toggle the sound." + NL +
" " + NL +
"Press \"Esc\" key to hide program." + NL +
" " + NL +
"Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program." + NL +
" " + NL +
"Unlike other puzzles in the collection there is no way to" + NL +
"move pieces without drag and drop or to randomize the puzzle." + NL,
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void referencesHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"References",
"Mark Manasse & Danny Sleator of AT&T Bell Laboratories " +
"and Victor K. Wei of Bell Communications Research," + NL +
"Some Results on the Panex Puzzle, Murray Hill, NJ, 1985 " +
"20 pp. (unpublished)." + NL +
"Vladimir Dubrovsky, Nesting Puzzles Part 1: Moving oriental " +
"towers, Quantum/Toy Store, January/February 1996 pp 55-57," + NL +
"50-51." + NL +
"L. E. Horden, Sliding Piece Puzzles (Recreations in " +
"Mathematics Series), Oxford University Press 1986," + NL +
"pp 144, 1." + NL +
"Jerry Slocum & Jack Botmans, Puzzles Old & New (How to Make " +
"and Solve Them), University of Washington" + NL +
"Press, Seattle 1987, p 135." + NL +
"Dick Hess, Analysis of the Algorithme 6 Puzzle and its " +
"Generalisations, Cubism For Fun, July 2008 76 pp 8-13." + NL,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void aboutHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"About",
"Java PanexFrame Version 8.7.2" + NL +
"Send bugs (reports or fixes) to the author:" +
"David Bagley <bagleyd AT verizon.net>" + NL +
"The latest version is at: " +
"https://www.sillycycle.com/puzzles.html" + NL,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public Icon getIcon(String string, int sx, int sy) {
		URL url = PanexFrame.class.getResource("/" + string);

		if (url == null)
			return null;
		return new Icon(Toolkit.getDefaultToolkit().
			getImage(url), sx, sy);
	}

	// TODO Make more reliable, also should cache
	public void playClip(String string) {
		URL url = PanexFrame.class.getResource("/" + string);

		if (url == null)
			return;
		try {
			AudioInputStream audioInputStream =
				AudioSystem.getAudioInputStream(url);
			DataLine.Info info = new DataLine.Info(Clip.class,
				audioInputStream.getFormat());
			Clip clip = (Clip)AudioSystem.getLine(info);
			clip.open(audioInputStream);
			clip.start();
		} catch (IOException | LineUnavailableException | UnsupportedAudioFileException e) {
			e.printStackTrace();
		}
	}

	public void playDripAudio() {
		playClip(PanexInterface.SOUNDS[PanexInterface.Sounds.DRIP.ordinal()]);
	}

	public void playMoveAudio() {
		playClip(PanexInterface.SOUNDS[PanexInterface.Sounds.MOVE.ordinal()]);
	}

	@Override
	public void setSize(int width, int height) {
		if (puzzle == null)
			return;
		puzzle.setBounds(0, 0, width, height);
		/*repaint();*/ // causes flashing
	}

	@Override
	public void paint(Graphics g) {
		puzzle.setSize(secondaryPanel.getSize().width,
			secondaryPanel.getSize().height - topPanel.getSize().height);
		super.paint(g);
	}

	public String readUserScore() {
		StringBuilder scoreString = new StringBuilder();

		for (int i = 0; i < puzzleRecord.length; i++) {
			for (int j = 0; j < puzzleRecord[i].length; j++) {
				scoreString.append(puzzleRecord[i][j]);
				if (j < puzzleRecord[i].length - 1)
					scoreString.append(" ");
			}
			if (i < puzzleRecord.length - 1)
				scoreString.append(NL);
		}
		return scoreString.toString();
	}

	public void writeScore() {
		try (PrintWriter pr = new PrintWriter(new FileWriter(
				SCOREFILE, false))) {
			pr.println(readUserScore());
		} catch (Exception e) {
			//e.printStackTrace();
		}
	}

	public void initRecords(String cookieValue, boolean store) {
		String[] st = null;
		int field = 0;

		if (cookieValue != null)
			st = cookieValue.split("[+/\\s]+");
		if (debug)
			System.out.println("initRecords: " + cookieValue);
		puzzleRecord = new int[PanexFormat.Modes.values().length][MAX_TILES];
		for (int i = 0; i < puzzleRecord.length; i++) {
		    for (int j = 0; j < puzzleRecord[i].length; j++) {
			if (st == null || st.length <= field) {
				//build it from the array
				puzzleRecord[i][j] =
					PanexScores.PUZZLE_SCORES[i][j];
			} else {
				puzzleRecord[i][j] =
					Integer.parseInt(st[field++]);
			}
		    }
		}
		if (store && st == null)
			writeScore(); // if no cookie, then make one...
	}

	public void readScoreRecords() {
		try (BufferedReader br = new BufferedReader(
				new FileReader(SCOREFILE))) {
			StringBuilder scoreData = new StringBuilder();
			String lineData;

			while ((lineData = br.readLine()) != null) {
				scoreData.append(lineData);
				scoreData.append(NL);
			}
			initRecords(scoreData.toString(), true);
		} catch (Exception e) {
			//e.printStackTrace();
			showMessage("Cannot open " + SCOREFILE);
			initRecords(null, true);
		}
	}

	public String readTextArea() {
		return ioDialog.getText();
	}

	public void writeTextArea(String s) {
		ioDialog.setText(s);
	}

	public void shuffleDown() {
		ComponentUtil.findJFrame(this).toBack();
	}

	public void quit() {
		System.exit(0);
	}

	void printRecord(int mode, int tile) {
		int i = mode;
		int j = tile - PanexFormat.MIN_TILES;

		if (puzzleRecord == null)
			return;
		if (tile > MAX_TILES) {
			recordValueLabel.setText("NONE");
		} else if (puzzleRecord[i][j] <= NEVER) {
			recordValueLabel.setText("NEVER");
		} else {
			recordValueLabel.setText(Integer.toString(
				puzzleRecord[i][j]));
		}
	}

	boolean handleSolved(int counter, int mode, int tile) {
		int i = mode;
		int j = tile - PanexFormat.MIN_TILES;

		if (puzzleRecord != null &&
				tile <= MAX_TILES &&
				(counter < puzzleRecord[i][j] ||
				puzzleRecord[i][j] <= NEVER)) {
			/* Maybe its been updated by another */
			readScoreRecords();
			puzzleRecord[i][j] = counter;
			writeScore();
			printRecord(mode, tile);
			return true;
		}
		return false;
	}

	public void initScore() {
		if (puzzleRecord == null) {
			readScoreRecords();
			printRecord(modeValue, tileValue);
		}
	}

	class TileChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = tileValue;

			tileValue = tileChanger.getValue();
			if (oldValue != tileValue) {
				puzzle.setPuzzleSize(tileValue);
				movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));
				printRecord(modeValue, tileValue);
			}
		}
	}

	class SpeedChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = speedValue;

			speedValue = speedChanger.getValue();
			if (oldValue != speedValue) {
				puzzle.setDelay(MAX_SPEED + MIN_SPEED - 1 -
					speedValue);
			}
		}
	}

	class ComboBoxActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent event) {
			int modeIndex = modeComboBox.getSelectedIndex();

			if (modeIndex != -1 && modeValue != modeIndex) {
				modeValue = modeIndex;
				puzzle.setPuzzleMode(modeValue);
				movesDisplay = 0;
				movesValueLabel.setText(Integer.toString(
					movesDisplay));
				printRecord(modeValue, tileValue);
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();

		for (int opt = 0; opt < COMMAND_STRINGS.length; opt++) {
			if (command.equals(COMMAND_STRINGS[opt])) {
				processKey(COMMAND_CHARS[opt]);
				return;
			}
		}
	}

	public void callback(int reason) {
		messageDisplay = "                                                                                ";
		switch(reason) {
		case PanexInterface.ACTION_RESTORE:
		case PanexInterface.ACTION_RESET:
			movesDisplay = 0;
			break;
		case PanexInterface.ACTION_ILLEGAL:
			messageDisplay = "Illegal move";
			break;
		case PanexInterface.ACTION_BLOCKED:
			messageDisplay = "Blocked";
			break;
		case PanexInterface.ACTION_SPACE:
			// Too annoying
			// messageDisplay = "A space cannot slide";
			break;
		case PanexInterface.ACTION_IGNORE:
			messageDisplay = "Clear to start";
			break;
		case PanexInterface.ACTION_MOVED:
			movesDisplay++;
			break;
		case PanexInterface.ACTION_SOLVED:
			if (puzzle.getCheat())
				messageDisplay = "No cheating!!"; //username
			else if (handleSolved(movesDisplay, modeValue, tileValue))
				messageDisplay = "Best solution, congratulations!!"; //username
			else
				messageDisplay = "Solved!";
			//showMessage(messageDisplay);
			break;
		case PanexInterface.ACTION_COMPUTED:
			break;
		case PanexInterface.ACTION_UNDO:
			movesDisplay--;
			break;
		case PanexInterface.ACTION_REDO:
			movesDisplay++;
			break;
		case PanexInterface.ACTION_INCREMENT:
			movesDisplay = 0;
			tileValue++;
			printRecord(modeValue, tileValue);
			puzzle.setPuzzleSize(tileValue);
			if (tileChanger != null)
				tileChanger.setValue(tileValue);
			break;
		case PanexInterface.ACTION_DECREMENT:
			movesDisplay = 0;
			tileValue--;
			printRecord(modeValue, tileValue);
			puzzle.setPuzzleSize(tileValue);
			if (tileChanger != null)
				tileChanger.setValue(tileValue);
			break;
		case PanexInterface.ACTION_MODE:
			movesDisplay = 0;
			modeValue++;
			if (modeValue >= PanexFormat.Modes.values().length)
				modeValue = PanexFormat.Modes.HANOI.ordinal();
			printRecord(modeValue, tileValue);
			puzzle.setPuzzleMode(modeValue);
			modeComboBox.setSelectedItem(
				PanexFormat.titleName(PanexFormat.Modes.values()[modeValue].toString()));
			break;
		case PanexInterface.ACTION_SPEED:
			{
				int oldDelay, delay;

				delay = puzzle.getDelay();
				oldDelay = delay;
				if (delay > MAX_SPEED - MIN_SPEED)
					delay = MAX_SPEED - MIN_SPEED;
				if (delay < 0)
					delay = 0;
				if (delay != oldDelay) {
					puzzle.setDelay(delay);
				}
				if (speedChanger != null)
					speedChanger.setValue(MAX_SPEED +
						MIN_SPEED - 1 - delay);
			}
			return;
		default:
		}
		messageLabel.setText(messageDisplay);
		movesValueLabel.setText(Integer.toString(movesDisplay));
	}

	public PanexFrame(String title) {
		/*JPanel info1Panel = new JPanel();
		JPanel leftPanel = new JPanel();
		JPanel rightPanel = new JPanel();*/
		JPanel startPanel = new JPanel();
		JPanel info2Panel = new JPanel();
		JPanel movesPanel = new JPanel();
		JPanel recordPanel = new JPanel();
		JPanel sliderPanel = new JPanel();
		JPanel comboBoxPanel = new JPanel();
		JPanel canvasPanel = new JPanel();

		try {
			mediumIcon = getIcon(PanexInterface.ICONS[PanexInterface.Icons.X32.ordinal()], 32, 32);
			largeIcon = getIcon(PanexInterface.ICONS[PanexInterface.Icons.X48.ordinal()], 48, 48);
		} catch (Exception e) {
			largeIcon = null;
		}
		ioDialog = new IODialog(ComponentUtil.findJFrame(this),
			PanexInterface.TITLE + ": IO", "Puzzle Position",
			"DONE");
		readScoreRecords();
		getOpts();
		speedValue = MAX_SPEED + MIN_SPEED - 1 - delayValue;
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBackground(Color.white);
		if (controlValue) {
			menuBar = new JMenuBar();
			JMenu menu;
			JMenuItem menuItem;

			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
			menu = new JMenu("File");
			menuItem = new JMenuItem("Get Buffer");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_G);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Write Buffer");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_W);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Show Buffer");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menu.addSeparator();
			menuItem = new JMenuItem("Get File");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Write File");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menu.addSeparator();
			menuItem = new JMenuItem("Exit");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_X);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Play");
			menuItem = new JMenuItem("Undo");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_U);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Redo");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_R);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Clear");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_C);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Auto-solve");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_S);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Mode (" +
				PanexFormat.Modes.HANOI.toString() + "/" +
				PanexFormat.Modes.ALGORITHME.toString() + "/" +
				PanexFormat.Modes.PANEX.toString() + ")");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_M);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Increment");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_I);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Decrement");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_D);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Speed >");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Slow <");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			soundMenuItem = new JCheckBoxMenuItem("Sound @", sound);
			menu.add(soundMenuItem);
			soundMenuItem.addActionListener(this);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Help");
			menuItem = new JMenuItem("Description ?");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Features !");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("References ^");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("About");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_A);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			//setJMenuBar(menuBar); // needs to be in a panel
			mainPanel.add(menuBar, BorderLayout.PAGE_START);
			/*leftPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			leftPanel.add(new JLabel("Move tile"));
			rightPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			rightPanel.add(new JLabel("Clear");
			leftPanel.add(getIcon("mouse-l.png", 16, 16));
			rightPanel.add(getIcon("mouse-r.png", 16, 16));
			info1Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 0));
			info1Panel.add(leftPanel);
			info1Panel.add(rightPanel);*/
			movesPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
			movesPanel.add(new JLabel("Moves"));
			movesValueLabel.setColumns(5);
			movesValueLabel.setEditable(false);
			movesPanel.add(movesValueLabel);
			recordPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
			recordPanel.add(new JLabel("Record"));
			recordValueLabel.setColumns(5);
			recordValueLabel.setEditable(false);
			recordPanel.add(recordValueLabel);
			info2Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 15, 5));
			info2Panel.add(movesPanel);
			info2Panel.add(recordPanel);
			sliderPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
			tileChanger = new Spinner("Size",
				tileValue, PanexFormat.MIN_TILES);
			/*tileChanger = new Slider("Size",
				tileValue, PanexFormat.MIN_TILES,
				((tileValue > MAX_TILES) ? tileValue : MAX_TILES));*/
			sliderPanel.add(tileChanger);
			speedChanger = new Spinner("Animation Speed",
				speedValue, MIN_SPEED,
				((speedValue > MAX_SPEED) ? speedValue : MAX_SPEED));
			/*speedChanger = new Slider("Animation Speed",
				speedValue, MIN_SPEED,
				((speedValue > MAX_SPEED) ? speedValue : MAX_SPEED));
			speedChanger.setWidth(150);*/
			sliderPanel.add(speedChanger);
			comboBoxPanel.setLayout(new FlowLayout(FlowLayout.LEFT,
				5, 5));
			for (PanexFormat.Modes mode : PanexFormat.Modes.values()) {
				modeComboBox.addItem(PanexFormat.titleName(mode.toString()));
			}
			modeComboBox.setSelectedItem(
				PanexFormat.titleName(PanexFormat.Modes.values()[modeValue].toString()));
			comboBoxPanel.add(modeComboBox);
			topPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.CENTER));
			/*topPanel.add(info1Panel);*/
			topPanel.add(info2Panel);
			topPanel.add(comboBoxPanel);
			topPanel.add(sliderPanel);
			topPanel.add(messageLabel);
			startPanel.setLayout(new ColumnLayout(0, 0, 0,
				ColumnLayout.LEFT));
			startPanel.add(topPanel);
			secondaryPanel.setLayout(new BorderLayout());
			secondaryPanel.add(startPanel, BorderLayout.PAGE_START);
		}
		puzzle = new Panex(this,
			modeValue, tileValue, false, false, monoString,
			pyramidColor[0], pyramidColor[1],
			tileColor, bgColor,
			MAX_SPEED + MIN_SPEED - 1 - speedValue);
		canvasPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		canvasPanel.add(puzzle);
		secondaryPanel.add(canvasPanel, BorderLayout.CENTER);
		mainPanel.add(secondaryPanel, BorderLayout.CENTER);
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		if (controlValue) {
			printRecord(modeValue, tileValue);
			tileChanger.addChangeListener(new TileChangeListener());
			modeComboBox.addActionListener(new ComboBoxActionListener());
			speedChanger.addChangeListener(new SpeedChangeListener());
		}
		canvasPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseEntered(MouseEvent event) {
				puzzle.requestFocus();
			}
		});
		setTitle(title);
		/* puzzle.setSize(getSize().width,
			getSize().height - topPanel.getSize().height); */
		puzzle.setSize(widthValue, heightValue);
	}

	public static void main(String[] args) {
		String laf = UIManager.getCrossPlatformLookAndFeelClassName();

		try {
			UIManager.setLookAndFeel(laf);
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e) {
			e.printStackTrace();
			System.exit(1);
		}
		List<Image> iconList = new ArrayList<>();
		for (String icon : PanexInterface.ICONS) {
			URL url = PanexFrame.class.getResource("/" + icon);
			if (url != null)
				iconList.add(Toolkit.getDefaultToolkit().getImage(url));
		}

		argumentParser = new ArgumentParser(args);
		if (argumentParser.hasOption("?")) {
			System.out.println("usage:" + NL +
"\t[-windowWidth={int}] [-windowHeight={int}]" + NL +
"\t[-windowX={int}] [-windowY={int}]" + NL +
"\t[-pyramidColor{0|1}={int}]" + NL +
"\t[-bgColor={int}] [-sound={bool}] [-control={bool}]" + NL +
"\t[-tiles={int}] [-mode={int}] [-delay={int}]");
			return;
		}
		PanexFrame frame = new PanexFrame(PanexInterface.TITLE);
		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		if (!iconList.isEmpty()) {
			frame.setIconImages(iconList);
		}
		frame.pack();
		frame.puzzle.requestFocusInWindow();
		frame.setLocation(PanexFrame.xValue, PanexFrame.yValue);
		frame.setVisible(true);
	}
}
